/*
 * The SmartWeb Framework Maven Plugin
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework Maven Plugin please visit
 *
 *                        http://smartweb.sourceforge.net/maven
 */
package net.smartlab.web.generator;

import java.beans.IntrospectionException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Writer;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import org.apache.maven.plugin.logging.Log;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 * @author gcontartese
 */
public class Main {

	/**
	 * Optional parameter: defaults to "etc/src/"
	 */
	public static final String SOURCE_DIRECTORY = "sourceOutputDirectory";

	/**
	 * Optional parameter: defaults to "etc/web/"
	 */
	public static final String WAR_SOURCE_DIRECTORY = "warSourceOutputDirectory";

	/**
	 * Optional parameter: defaults to "generated"
	 */
	public static final String BUNDLE = "bundle";

	/**
	 * Required parameter
	 */
	public static final String CLASS_LIST = "classes";

	/**
	 * Required parameter
	 */
	public static final String PACKAGE = "package";

	protected Log logger;

	private Properties defaults = new Properties();

	private Collection classes = new HashSet();


	public static void main(String[] args) throws Exception {
		Main main = new Main();
		main.generateAll();
	}

	public Main() throws IOException {
		try {
			defaults.load(Main.class.getResourceAsStream("defaults.properties"));
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}

	protected void init() throws Exception {
		Velocity.setProperty("resource.loader", "classpath");
		Velocity.setProperty("classpath.resource.loader.class",
				"org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
		Velocity.init();
		if (classes.isEmpty()) {
			if (this.getProperty(CLASS_LIST) != null) {
				this.setClasses(this.getProperty(CLASS_LIST));
			}
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @param args
	 */
	public void generateAll() throws Exception {
		this.generateFactories();
		this.generateDomain();
		this.generateActions();
		this.generateStruts();
		this.generatePages();
	}

	/**
	 * TODO documentation
	 * 
	 * @throws Exception
	 */
	public void generateDomain() throws Exception {
		this.init();
		String path = this.getProperty(SOURCE_DIRECTORY) + this.getProperty(PACKAGE).replace('.', '/') + '/';
		new File(path).mkdirs();
		VelocityContext global = new VelocityContext();
		global.put("classes", this.getClasses());
		global.put("package", this.getProperty(PACKAGE));
		this.info("\tgenerated Domain.java");
		this.merge(Velocity.getTemplate("/META-INF/domain.vm"), global, path + "Domain.java");
	}

	/**
	 * TODO documentation
	 * 
	 * @throws Exception
	 */
	public void generateFactories() throws Exception {
		this.init();
		String path = this.getProperty(SOURCE_DIRECTORY) + this.getProperty(PACKAGE).replace('.', '/') + '/';
		new File(path).mkdirs();
		VelocityContext global = new VelocityContext();
		global.put("classes", this.getClasses());
		global.put("package", this.getProperty(PACKAGE));
		Template factory = Velocity.getTemplate("/META-INF/factory.vm");
		Iterator iterator = this.getClasses().iterator();
		while (iterator.hasNext()) {
			Type type = (Type)iterator.next();
			VelocityContext context = new VelocityContext(global);
			context.put("class", type);
			this.info("\tgenerated " + type.getName() + "Factory.java");
			this.merge(factory, context, path + type.getName() + "Factory.java");
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @throws Exception
	 */
	public void generateActions() throws Exception {
		this.init();
		String path = this.getProperty(SOURCE_DIRECTORY) + this.getProperty(PACKAGE).replace('.', '/') + '/';
		new File(path).mkdirs();
		VelocityContext global = new VelocityContext();
		global.put("classes", this.getClasses());
		global.put("package", this.getProperty(PACKAGE));
		Template action = Velocity.getTemplate("/META-INF/action.vm");
		Iterator iterator = this.getClasses().iterator();
		while (iterator.hasNext()) {
			Type type = (Type)iterator.next();
			VelocityContext context = new VelocityContext(global);
			context.put("class", type);
			this.info("\tgenerating " + type.getName() + "Action.java");
			this.merge(action, context, path + type.getName() + "Action.java");
		}
	}

	/**
	 * TODO documentation
	 * 
	 * @throws Exception
	 */
	public void generateStruts() throws Exception {
		this.init();
		new File(this.getProperty(WAR_SOURCE_DIRECTORY) + "WEB-INF/").mkdirs();
		VelocityContext global = new VelocityContext();
		global.put("classes", this.getClasses());
		global.put("package", this.getProperty(PACKAGE));
		global.put("bundle", this.getProperty(BUNDLE));
		this.info("\tgenerated struts.xml");
		this.merge(Velocity.getTemplate("/META-INF/struts.vm"), global, this.getProperty(WAR_SOURCE_DIRECTORY)
				+ "WEB-INF/struts.xml");
		this.info("\tgenerated form.xml");
		this.merge(Velocity.getTemplate("/META-INF/forms.vm"), global, this.getProperty(WAR_SOURCE_DIRECTORY)
				+ "WEB-INF/forms.xml");
	}

	/**
	 * TODO documentation
	 * 
	 * @throws Exception
	 */
	public void generatePages() throws Exception {
		this.init();
		// Creates paths
		String path = this.getProperty(SOURCE_DIRECTORY) + this.getProperty(PACKAGE).replace('.', '/') + '/';
		new File(path).mkdirs();
		new File(this.getProperty(WAR_SOURCE_DIRECTORY) + "WEB-INF/").mkdirs();
		// Generate pages
		VelocityContext global = new VelocityContext();
		global.put("classes", this.getClasses());
		global.put("package", this.getProperty(PACKAGE));
		global.put("bundle", this.getProperty(BUNDLE));
		this.merge(Velocity.getTemplate("/META-INF/messages.vm"), global, path + "messages.properties");
		Template editJSP = Velocity.getTemplate("/META-INF/edit.jsp.vm");
		Template listJSP = Velocity.getTemplate("/META-INF/list.jsp.vm");
		Template viewJSP = Velocity.getTemplate("/META-INF/view.jsp.vm");
		Iterator iterator = this.getClasses().iterator();
		while (iterator.hasNext()) {
			Type type = (Type)iterator.next();
			VelocityContext context = new VelocityContext(global);
			context.put("class", type);
			this.info("\tgenerated " + type.getInstance() + "-edit.jsp");
			this.merge(editJSP, context, this.getProperty(WAR_SOURCE_DIRECTORY) + type.getInstance() + "-edit.jsp");
			this.info("\tgenerated " + type.getInstance() + "-list.jsp");
			this.merge(listJSP, context, this.getProperty(WAR_SOURCE_DIRECTORY) + type.getInstance() + "-list.jsp");
			this.info("\tgenerated " + type.getInstance() + "-view.jsp");
			this.merge(viewJSP, context, this.getProperty(WAR_SOURCE_DIRECTORY) + type.getInstance() + "-view.jsp");
		}
		// Extract included and used files
		this.extract();
	}

	private Collection getClasses() throws IntrospectionException, ClassNotFoundException {
		return classes;
	}

	public void setClasses(Set classes) throws IntrospectionException {
		Iterator iterator = classes.iterator();
		while (iterator.hasNext()) {
			this.classes.add(new Type((Class)iterator.next()));
		}
	}

	public void setClasses(String[] classes) throws IntrospectionException, ClassNotFoundException {
		for (int i = 0; i < classes.length; i++) {
			this.classes.add(new Type(Class.forName(classes[i])));
		}
	}

	public void setClasses(String classes) throws IntrospectionException, ClassNotFoundException {
		ArrayList list = new ArrayList();
		StringTokenizer tokenizer = new StringTokenizer(classes, ",");
		while (tokenizer.hasMoreTokens()) {
			list.add(tokenizer.nextToken().trim());
		}
		this.setClasses((String[])list.toArray(new String[list.size()]));
	}

	private void merge(Template template, VelocityContext context, String file) throws IOException {
		Writer writer = new FileWriter(file);
		template.merge(context, writer);
		writer.close();
	}

	public String getProperty(String key) {
		String value = System.getProperty(key, defaults.getProperty(key));
		if (key.endsWith("Directory") && !value.endsWith("/")) {
			value = value + "/";
			System.setProperty(key, value);
		}
		return value;
	}

	public void setLog(org.apache.maven.plugin.logging.Log logger) {
		this.logger = logger;
	}

	private void info(String message) {
		if (logger != null) {
			logger.info(message);
		} else {
			System.out.println(message);
		}
	}

	private void extract() throws FileNotFoundException, IOException {
		String RES = "META-INF/resources/";
		this.info("\textracting resources");
		URL url = this.getClass().getProtectionDomain().getCodeSource().getLocation();
		if (!new File(url.getFile()).isDirectory()) {
			JarInputStream in = new JarInputStream(new FileInputStream(url.getFile()));
			byte[] buffer = new byte[1024];
			for (JarEntry entry = in.getNextJarEntry(); entry != null; entry = in.getNextJarEntry()) {
				if (entry.getName().startsWith(RES)) {
					if (!entry.isDirectory()) {
						String filename = entry.getName().substring(RES.length());
						OutputStream out = new FileOutputStream(new File(this.getProperty(WAR_SOURCE_DIRECTORY),
								filename));
						for (int read = in.read(buffer, 0, buffer.length); read > -1; read = in.read(buffer, 0,
								buffer.length)) {
							out.write(buffer, 0, read);
						}
						out.close();
					} else {
						File path = new File(this.getProperty(WAR_SOURCE_DIRECTORY)
								+ entry.getName().substring(RES.length()));
						path.mkdirs();
					}
				}
			}
		}
	}
}
