/*
 * The SmartWeb Framework Maven Plugin
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework Maven Plugin please visit
 *
 *                        http://smartweb.sourceforge.net/maven
 */
package net.smartlab.web.generator;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Array;

import net.smartlab.web.BusinessObject;
import net.smartlab.web.Enumeration;
import net.smartlab.web.StringEnumeration;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 */
public class Property {

	private PropertyDescriptor descriptor;


	public Property(PropertyDescriptor descriptor) {
		this.descriptor = descriptor;
	}

	public String getName() {
		return descriptor.getName();
	}

	public boolean isSimple() {
	    if(descriptor.getPropertyType()!=null){
		return descriptor.getPropertyType().equals(String.class) || descriptor.getPropertyType().equals(Boolean.class)
				|| descriptor.getPropertyType().equals(Byte.class) || descriptor.getPropertyType().equals(Short.class)
				|| descriptor.getPropertyType().equals(Integer.class)
				|| descriptor.getPropertyType().equals(Long.class) || descriptor.getPropertyType().equals(Float.class)
				|| descriptor.getPropertyType().equals(Double.class);
	    }
	    return false;
	}

	public boolean isDateTime() {
	    if(descriptor.getPropertyType()!=null){
	        return java.util.Date.class.isAssignableFrom(descriptor.getPropertyType()) && !(this.isDate() || this.isTime());
	    }
	    return false;
	}

	public boolean isDate() {
	    if(descriptor.getPropertyType()!=null){
	        return java.sql.Date.class.isAssignableFrom(descriptor.getPropertyType());
	    }
	    return false;
	}

	public boolean isTime() {
	    if(descriptor.getPropertyType()!=null){
	        return java.sql.Time.class.isAssignableFrom(descriptor.getPropertyType());
	    }
	    return false;
	}

	public boolean isCollection() {
	    if(descriptor.getPropertyType()!=null){
		return java.util.Collection.class.isAssignableFrom(descriptor.getPropertyType())
				|| java.util.Map.class.isAssignableFrom(descriptor.getPropertyType())
				|| java.util.Iterator.class.isAssignableFrom(descriptor.getPropertyType())
				|| descriptor.getPropertyType().equals(Array.class);
	    }
	    return false;
	}

	public boolean isMap() {
	    if(descriptor.getPropertyType()!=null){
	        return java.util.Map.class.isAssignableFrom(descriptor.getPropertyType());
	    }
	    return false;
	}

	public boolean isAssociation() {
	    if(descriptor.getPropertyType()!=null){
	        return BusinessObject.class.isAssignableFrom(descriptor.getPropertyType());
	    }
	    return false;
	}

	public boolean isEnumeration() {
	    if(descriptor.getPropertyType()!=null){
		return Enumeration.class.isAssignableFrom(descriptor.getPropertyType())
				|| StringEnumeration.class.isAssignableFrom(descriptor.getPropertyType());
	    }
	    return false;
	}
	/*
	 * private void test() { ResourceBundle data =
	 * LocaleData.getDateFormatData(Locale.UK); java.util.Enumeration
	 * enumeration = data.getKeys(); while (enumeration.hasMoreElements()) {
	 * String key = (String)enumeration.nextElement(); System.out.print(key +
	 * " = "); Object value = data.getObject(key); if (value instanceof String)
	 * { System.out.println(value); } else { String[] values = (String[])value;
	 * for (int i = 0; i < values.length; i++) { System.out.print(values[i] +
	 * ", "); } System.out.println(); } } System.exit(0); String[] stamps =
	 * LocaleData
	 * .getDateFormatData(Locale.CHINA).getStringArray("DateTimePatterns"); int
	 * TIME = 3; int DATE = 7; char timeSep = this.getSeparator(stamps[TIME]);
	 * char dateSep = this.getSeparator(stamps[DATE]);
	 * System.out.println(timeSep + " " + dateSep);
	 * DateFormatSymbols.getInstance(Locale.ITALY); }
	 */
	/*
	 * private char getSeparator(String format) { char[] chars =
	 * format.toCharArray(); for (int i = 0; i < chars.length; i++) { if
	 * (!(Character.isSpaceChar(chars[i]) ||
	 * Character.isLetterOrDigit(chars[i]))) { return chars[i]; } } throw new
	 * RuntimeException(); }
	 */
}
